//*****************************************************************************
//*****************************************************************************
//  FILENAME:  `@LIB_NAME`_PULSELED.c
//  Version: 1.0.0.2, Updated on 2006/08/01 at 15:25:26
//  Application: PSoC Express 2.0
//
//  DESCRIPTION:  This Driver controls the PULSELED parameters and returns
//                the percent duty cycle for the selected channel.
//-----------------------------------------------------------------------------
//      Copyright (c) Cypress MicroSystems 2006. All Rights Reserved.
//*****************************************************************************
//*****************************************************************************

#include "m8c.h"
#include "`@LIB_NAME`_PULSELED.h"
#include "cmx.h"
#include "SystemTimer.h"
// Channel type header file
#include "CMX_PWM_CHAN.h"

#define  FACTOR 100

#define CMX_DECAY_OFF        0    // LED output decay off.
#define CMX_GROW_ON          1    // LED output grow  on.
#define STM_LED_ON           2    // LED output fully on.
#define STM_LED_OFF          3    // LED output fully off.

// Start time for each instance of driver in System Timer ticks
unsigned int uPULSELEDCount[CMX_PULSELED_COUNT];  
// Array for saving previous state of LED output for each instance of driver
BYTE     bPrevPULSELEDState[CMX_PULSELED_COUNT];  


//-----------------------------------------------------------------------------
//  FUNCTION NAME: `@LIB_NAME`_PULSELED_Instantiate(const `@LIB_NAME`_PULSELED_ParameterBlock * thisBLK)
//
//  DESCRIPTION: PULSELED Initialization
//
//-----------------------------------------------------------------------------
//
//  ARGUMENTS: 
//    thisBLK  => Pointer to device parameter block
//
//  RETURNS: None
//
//  SIDE EFFECTS: 
//
//  THEORY of OPERATION or PROCEDURE:
//    Initialization of PULSELED. Initial state set to STM_LED_OFF.
//
////////////////////////////////////////////////////////////////////////////////

void `@LIB_NAME`_PULSELED_Instantiate(const `@LIB_NAME`_PULSELED_ParameterBlock * thisBLK)
{
    BYTE bInstance;

    bInstance = thisBLK->PULSELED_ChannelID;
    bPrevPULSELEDState[thisBLK->PULSELED_ID] = STM_LED_OFF;

    PWMSetPolarity(bInstance,1);
    PWMSetDutyCycle(bInstance, 0);
    PWMStart(bInstance);
}
//-----------------------------------------------------------------------------
//  FUNCTION NAME: `@LIB_NAME`_PULSELED_SetValue(`@LIB_NAME`_PULSELED_ParameterBlock * thisBLK, BYTE bState)
//
//  DESCRIPTION: Sets the PWM duty cycle percent value
//
//-----------------------------------------------------------------------------
//
//  ARGUMENTS: 
//    thisBLK       => Pointer to parameter block
//    bState        => State of LED (GROW_ON or DECAY_OFF)
//
//  RETURNS: None
//
//  SIDE EFFECTS: 
//
//  THEORY of OPERATION:
//    LED output depends on bState, when it is GROW_ON the output will grow 
//    exponentially until fully on, when it is DECAY_OFF it will decay 
//    exponentially until fully off, based upon the decay/grow time property.
//
////////////////////////////////////////////////////////////////////////////////

void `@LIB_NAME`_PULSELED_SetValue(const `@LIB_NAME`_PULSELED_ParameterBlock * thisBLK, BYTE bState)
{
    BYTE bID;						// Driver instance
    BYTE bPULSELEDDutyCycle;		// Output PWM signal duty cycle
	
    unsigned int uDecayGrowTime;	// Decay/Grow Time in ticks
	unsigned int uTicks;			// Current time in ticks from system timer
    unsigned int uDT;				// Time left to final (fully on/fully off) state


    bID = thisBLK->PULSELED_ID;		// Get instance number

    uDecayGrowTime = thisBLK->uLED_Decay_Grow_Time;	// Get Decay/Grow Time
    uTicks = SystemTimer_iGetTickCntr();			// Get current system timer time

    if (bState == CMX_GROW_ON)		// If output must grow
    {
        switch(bPrevPULSELEDState[bID])
        {        
	        case STM_LED_ON:	// if already in fully on state do nothing
	            bPULSELEDDutyCycle = 100;
	            break;

			case STM_LED_OFF:	// If previous state was fully off, start to grow
	            bPrevPULSELEDState[bID] = CMX_GROW_ON;
	            uPULSELEDCount[bID] = uTicks;		// Initialize start time
	            bPULSELEDDutyCycle = 0;
	            break;

			case CMX_DECAY_OFF:	// If state changed from decay to grow modify the start time 
				bPrevPULSELEDState[bID] = CMX_GROW_ON; 
				uDT = uTicks - uPULSELEDCount[bID];             
				if(uDT<uDecayGrowTime)
					uPULSELEDCount[bID] = uTicks + uDT - uDecayGrowTime;
				else
				    uPULSELEDCount[bID]= uTicks;
				    
	        case CMX_GROW_ON:	// If already in GROW_ON then we continue to grow
	            uDT = uTicks - uPULSELEDCount[bID];	//get duration time in ticks
	            if (uDT <= uDecayGrowTime) 
	            {   // Grow the LED
	                bPULSELEDDutyCycle = ((FACTOR*((long)uDT)))/uDecayGrowTime;  
	                
	            } 
	            else // If time exceeded turn to fully on state
	            {   
	                bPrevPULSELEDState[bID] = STM_LED_ON; 
	                bPULSELEDDutyCycle = 100;  
	            }           
	            break ;    
        }           
    } 
    else // if (bState == CMX_DECAY_OFF) if output must decay
    {
        switch(bPrevPULSELEDState[bID])
        {        
	        case STM_LED_ON:	// if LED was fully lit, start to decay
	            bPrevPULSELEDState[bID] = CMX_DECAY_OFF;
	            uPULSELEDCount[bID] = uTicks;        
	            bPULSELEDDutyCycle=100;
	            break;
	
	        case STM_LED_OFF:	// if already in fully off state do nothing
	            bPULSELEDDutyCycle = 0;        
	            break;
	
	        case CMX_GROW_ON:	// If state changed from grow to decay modify the start time       
				bPrevPULSELEDState[bID] = CMX_DECAY_OFF; 
				uDT = uTicks - uPULSELEDCount[bID];
				if(uDT<uDecayGrowTime)
					uPULSELEDCount[bID] = uTicks + uDT - uDecayGrowTime;            
				else
				    uPULSELEDCount[bID]= uTicks;
	
	        case CMX_DECAY_OFF:	// If already in CMX_DECAY_OFF then we continue to decay
	            uDT = uTicks - uPULSELEDCount[bID];   //get duration  time in ticks
	            if (uDT <= uDecayGrowTime) 
	            {   // Decay the LED
	                bPULSELEDDutyCycle = 100 - ((FACTOR*((long)uDT)))/uDecayGrowTime;   
	            } 
	            else // If time exceeded turn to fully off state
	            {   
	                bPrevPULSELEDState[bID] = STM_LED_OFF;
	                bPULSELEDDutyCycle = 0;
	            }                    
	            break;
        }           
    }
    // Control the LED intensity by setting the PWM signal duty cycle
    PWMSetDutyCycle(thisBLK->PULSELED_ChannelID, bPULSELEDDutyCycle);
}
